@@ -56,6 +56,7 @@ gchar* rb_search_fold (const char *original);
 gchar** rb_string_split_words (const gchar *string);
 
 gboolean rb_string_list_equal (GList *a, GList *b);
+gboolean rb_string_list_contains (GList *list, const char *s);
 void rb_list_deep_free (GList *list);
 GList* rb_string_list_copy (GList *list);
 
@@ -74,6 +75,8 @@ gboolean rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
 void rb_value_array_append_data (GValueArray *array, GType type, ...);
 void rb_value_free (GValue *val);
 
+void rb_assert_locked (GMutex *mutex);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
