@@ -38,6 +38,7 @@ static void rb_shell_clipboard_class_init (RBShellClipboardClass *klass);
 static void rb_shell_clipboard_init (RBShellClipboard *shell_clipboard);
 static GObject *rb_shell_clipboard_constructor (GType type, guint n_construct_properties,
 						GObjectConstructParam *construct_properties);
+static void rb_shell_clipboard_dispose (GObject *object);
 static void rb_shell_clipboard_finalize (GObject *object);
 static void rb_shell_clipboard_set_property (GObject *object,
 					     guint prop_id,
@@ -180,6 +181,7 @@ rb_shell_clipboard_class_init (RBShellClipboardClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = rb_shell_clipboard_dispose;
 	object_class->finalize = rb_shell_clipboard_finalize;
 	object_class->constructor = rb_shell_clipboard_constructor;
 
@@ -263,6 +265,36 @@ unset_source_internal (RBShellClipboard *clipboard)
 	clipboard->priv->source = NULL;
 }
 
+static void
+rb_shell_clipboard_dispose (GObject *object)
+{
+	RBShellClipboard *shell_clipboard;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_SHELL_CLIPBOARD (object));
+
+	shell_clipboard = RB_SHELL_CLIPBOARD (object);
+
+	g_return_if_fail (shell_clipboard->priv != NULL);
+
+	unset_source_internal (shell_clipboard);
+
+	if (shell_clipboard->priv->idle_sync_id != 0) {
+		g_source_remove (shell_clipboard->priv->idle_sync_id);
+		shell_clipboard->priv->idle_sync_id = 0;
+	}
+	if (shell_clipboard->priv->idle_deletion_id != 0) {
+		g_source_remove (shell_clipboard->priv->idle_deletion_id);
+		shell_clipboard->priv->idle_deletion_id = 0;
+	}
+	if (shell_clipboard->priv->idle_playlist_id != 0) {
+		g_source_remove (shell_clipboard->priv->idle_playlist_id);
+		shell_clipboard->priv->idle_playlist_id = 0;
+	}
+
+	G_OBJECT_CLASS (rb_shell_clipboard_parent_class)->dispose (object);
+}
+
 static void
 rb_shell_clipboard_finalize (GObject *object)
 {
@@ -275,9 +307,6 @@ rb_shell_clipboard_finalize (GObject *object)
 
 	g_return_if_fail (shell_clipboard->priv != NULL);
 
-	/* release references to the source */
-	unset_source_internal (shell_clipboard);
-
 	g_hash_table_destroy (shell_clipboard->priv->signal_hash);
 
 	g_list_foreach (shell_clipboard->priv->entries, (GFunc)rhythmdb_entry_unref, NULL);
@@ -285,13 +314,6 @@ rb_shell_clipboard_finalize (GObject *object)
 
 	g_async_queue_unref (shell_clipboard->priv->deleted_queue);
 
-	if (shell_clipboard->priv->idle_sync_id)
-		g_source_remove (shell_clipboard->priv->idle_sync_id);
-	if (shell_clipboard->priv->idle_deletion_id)
-		g_source_remove (shell_clipboard->priv->idle_deletion_id);
-	if (shell_clipboard->priv->idle_playlist_id)
-		g_source_remove (shell_clipboard->priv->idle_playlist_id);
-
 	G_OBJECT_CLASS (rb_shell_clipboard_parent_class)->finalize (object);
 }
 
@@ -509,13 +531,12 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	GtkAction *action;
 	RhythmDBEntryType entry_type;
 
-	if (!clipboard->priv->source)
-		return;
-
-	view = rb_source_get_entry_view (clipboard->priv->source);
-	if (view) {
-		have_selection = rb_entry_view_have_selection (view);
-		can_select_all = !rb_entry_view_have_complete_selection (view);
+	if (clipboard->priv->source) {
+		view = rb_source_get_entry_view (clipboard->priv->source);
+		if (view) {
+			have_selection = rb_entry_view_have_selection (view);
+			can_select_all = !rb_entry_view_have_complete_selection (view);
+		}
 	}
 
 	if (clipboard->priv->queue_source) {
@@ -527,7 +548,7 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 
 	rb_debug ("syncing clipboard");
 
-	if (g_list_length (clipboard->priv->entries) > 0)
+	if (clipboard->priv->source != NULL && g_list_length (clipboard->priv->entries) > 0)
 		can_paste = rb_source_can_paste (clipboard->priv->source);
 
 	if (have_selection) {
@@ -578,12 +599,16 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
 
 	/* disable the whole add-to-playlist menu if we can't add to a playlist
-	 * FIXME: change this when we support non-library playilst adding
+	 * FIXME: change this when we support non-library playlist adding
 	 */
 	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditPlaylistAdd");
-	g_object_get (clipboard->priv->source, "entry-type", &entry_type, NULL);
-	gtk_action_set_sensitive (action, (entry_type == RHYTHMDB_ENTRY_TYPE_SONG));
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	if (clipboard->priv->source != NULL) {
+		g_object_get (clipboard->priv->source, "entry-type", &entry_type, NULL);
+		gtk_action_set_sensitive (action, (entry_type == RHYTHMDB_ENTRY_TYPE_SONG));
+		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	} else {
+		gtk_action_set_sensitive (action, FALSE);
+	}
 }
 
 static GtkWidget*
